@@ -235,6 +235,18 @@ class BaseField(object):
         instance._kwargs = kwargs
         return instance
 
+    def create(self, validated_data):
+        model_class = self.Meta.model
+
+        ids = ['id', 'pk', '_id']
+        for id_field in ids:
+            validated_data.pop(id_field, None)
+
+        return model_class.objects.create(**validated_data)
+
+    def save(self):
+        return self.create(self.validated_data)
+
 
 class BooleanField(BaseField):
     TRUE_VALUES = ['t', 'T', 'true', 'True', 'TRUE', '1', 1, True]
