@@ -1,7 +1,9 @@
 import copy
+import re
 from functools import cached_property
 
-from google.protobuf.json_format import ParseDict # noqa
+from google.protobuf.json_format import ParseDict, MessageToDict  # noqa
+from google.protobuf.message import Message # noqa
 
 from mgo_serializer.fields import BaseField, Empty, set_value
 from mgo_serializer.exceptions import ValidationError
@@ -19,6 +21,24 @@ def mongo_to_protobuf(data, schema):
     return ParseDict(data, protobuf)
 
 
+def dict_to_lower_case_keys(d):
+    to_dict = {}
+    for key, value in d.items():
+        parse = re.findall(r'[A-Z]?[a-z]+', key)
+        key = '_'.join([item.lower() for item in parse])
+        to_dict[key] = value
+    return to_dict
+
+
+def proto_to_dict(data):
+    data = MessageToDict(data)
+    for key, value in data.items():
+        if isinstance(value, dict):
+            data[key] = dict_to_lower_case_keys(value)
+
+    return data
+
+
 class BaseSerializer(BaseField):
     """
     Base class for all serializers.
@@ -26,8 +46,24 @@ class BaseSerializer(BaseField):
 
     def __init__(self, instance=None, data=Empty, **kwargs):
         self.instance = instance
+
+        if isinstance(self.instance, Message):
+            self.instance = proto_to_dict(self.instance)
+        elif isinstance(self.instance, list) and any(isinstance(item, Message) for item in self.instance):
+            self.instance = [proto_to_dict(item) for item in self.instance]
+        else:
+            self.instance = copy.deepcopy(self.instance)
+
         if data is not Empty:
-            self.initial_data = data
+            if isinstance(data, list) and any(isinstance(item, Message) for item in data):
+                self.initial_data = [proto_to_dict(item) for item in data]
+            elif isinstance(data, list) and not any(isinstance(item, Message) for item in data):
+                self.initial_data = data
+            elif isinstance(data, Message):
+                self.initial_data = proto_to_dict(data)
+            else:
+                self.initial_data = data
+
         self.partial = kwargs.pop("partial", False)
         kwargs.pop("many", None)
         super().__init__(**kwargs)
@@ -65,7 +101,7 @@ class BaseSerializer(BaseField):
     def to_protobuf(self, data):
         raise NotImplementedError('.to_protobuf() must be implemented.')
 
-    def is_valid(self):
+    def is_valid(self, raise_exception=False):
         """
         Validates the data.
         """
@@ -78,6 +114,9 @@ class BaseSerializer(BaseField):
             else:
                 self._errors = {}
 
+        if self._errors and raise_exception:
+            raise ValidationError(self._errors)
+
         return not bool(self._errors)
 
     @property
@@ -208,11 +247,11 @@ class ListSerializer(BaseSerializer):
             value = self.validate(value)
             assert value is not None, '.validate() should return the validated data'
         except ValidationError:
-            raise ValidationError('Validation error')
+            raise ValidationError(detail='_')
 
         return value
 
-    def is_valid(self):
+    def is_valid(self, raise_exception=False):
         """
         This implementation is the same as the default, but we use lists as default data instead of dicts.
         """
@@ -225,6 +264,9 @@ class ListSerializer(BaseSerializer):
             else:
                 self._errors = []
 
+        if self._errors and raise_exception:
+            raise ValidationError(self._errors)
+
         return not bool(self._errors)
 
     @property
@@ -268,9 +310,9 @@ class Serializer(BaseSerializer, metaclass=SerializerMetaclass):
     def get_initial(self):
         if hasattr(self, 'initial_data'):
             return dict([
-                (field_name, field.get_value(self.initial_data))
+                (field_name, field.get_value(self.initial_data, self))
                 for field_name, field in self.fields.items()
-                if (field.get_value(self.initial_data) is not Empty)
+                if (field.get_value(self.initial_data, self) is not Empty)
             ])
 
         return dict([
@@ -291,10 +333,13 @@ class Serializer(BaseSerializer, metaclass=SerializerMetaclass):
                 if validate_method:
                     validated_value = validate_method(primitive_value)
             except ValidationError as e:
-                errors[field.field_name] = 'ERROR: ' + str(e)
+                errors[field.field_name] = e.detail
             else:
                 set_value(ret, field.attributes, validated_value)
 
+        if errors:
+            raise ValidationError(errors)
+
         return ret
 
     def to_representation(self, instance):
